﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Aspose.Words;
using System.IO;
using System.Drawing;
using Aspose.Words.Drawing;
using Aspose.Words.Tables;

namespace OperateWordPro.OperateWordDemo2
{
    public partial class OperateWordByAspose : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        #region 1.0-基本功能

        /// <summary>
        /// 1.1 创建一个word
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void CreateWord_Click(object sender, EventArgs e)
        {
            // 实例化 Aspose.Words.Document 对象
            Document doc = new Document();

            // 实例化 Aspose.Words.DocumentBuilder 对象 （后面几乎都是通过操作DocumentBuilder实现输出word内容的
            DocumentBuilder builder = new DocumentBuilder(doc);

            // 往word中输入内容
            builder.Writeln("这是创建的一个word文档");

            // 保存到指定的路径
            doc.Save("G:/CreateWord(" + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss") + ").doc");
        }

        /// <summary>
        /// 1.2 加载一个word文档
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void LoadWord_Click(object sender, EventArgs e)
        {
            // 找到项目中的word模板 并转换成物理路径
            var path = Server.MapPath("../Document/AsposeWord.doc");

            // 实例化 Aspose.Words.Document 对象
            Document doc = new Document(path);

            // 实例化 Aspose.Words.DocumentBuilder 对象 （后面几乎都是通过操作DocumentBuilder实现输出word内容的
            DocumentBuilder builder = new DocumentBuilder(doc);

            // 控制显示的形式
            doc.ViewOptions.ViewType = ViewType.PageLayout;

            // 控制显示的大小比例此处为百分比，范围是0~100
            doc.ViewOptions.ZoomPercent = 100;

            // 往word中输入内容
            builder.Writeln("这是加载的word文档");

            // 文件保存到客户端
            var fileName = "LoadWord(" + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss") + ").doc";

            // 保存到客户端
            doc.Save(fileName, SaveFormat.Doc, SaveType.OpenInBrowser, Response);
            //doc.Save("G:/LoadWord(" + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss") + ").doc");

        }

        /// <summary>
        /// 1.3 控制文字输出样式
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void CreateWordContainFontStyle_Click(object sender, EventArgs e)
        {
            var path = Server.MapPath("../Document/AsposeWord.doc");
            Document doc = new Document(path);
            DocumentBuilder builder = new DocumentBuilder(doc);

            Aspose.Words.Font font = builder.Font;
            font.Size = 16;
            font.Bold = true;
            font.Color = Color.Blue;
            font.Name = "Arial";
            font.Underline = Underline.Dash;
            builder.Write("文字样式控制");

            var fileName = "CreateWordContainFontStyle(" + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss") + ").doc";
            doc.Save(fileName, SaveFormat.Doc, SaveType.OpenInBrowser, Response);
        }

        /// <summary>
        ///  1.4 替换word中的站位
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ReplaceFlag_Click(object sender, EventArgs e)
        {
            var path = Server.MapPath("../Document/AsposeWord2.doc");
            Document doc = new Document(path);
            DocumentBuilder builder = new DocumentBuilder(doc);

            doc.Range.Replace("{name}", "Little-Ant", false, false);
            doc.Range.Replace("{city}", "上海，徐汇", false, false);

            var fileName = "ReplaceFlag(" + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss") + ").doc";
            doc.Save(fileName, SaveFormat.Doc, SaveType.OpenInBrowser, Response);
        }

        /// <summary>
        /// 1.5 添加水印
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void WaterMark_Click(object sender, EventArgs e)
        {
            var path = Server.MapPath("../Document/AsposeWord2.doc");
            Document doc = new Document(path);
            DocumentBuilder builder = new DocumentBuilder(doc);

            doc.Range.Replace("{name}", "Little-Ant", false, false);
            doc.Range.Replace("{city}", "上海，徐汇", false, false);

            // 添加水印
            InsertWatermarkText(doc, "http://www.cnblogs.com/littleAnt-strongPower/");

            var fileName = "WaterMark(" + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss") + ").doc";
            doc.Save(fileName, SaveFormat.Doc, SaveType.OpenInBrowser, Response);
        }
        #region 1.5.1 添加水印

        public void InsertWatermarkText(Document doc, string watermarkText)
        {
            // Create a watermark shape. This will be a WordArt shape. 
            // You are free to try other shape types as watermarks.
            Shape watermark = new Shape(doc, ShapeType.TextPlainText);
            // Set up the text of the watermark.
            watermark.TextPath.Text = watermarkText;
            watermark.TextPath.FontFamily = "Arial";

            // 控制水印的大小
            watermark.Width = 500;
            watermark.Height = 60;

            // 倾斜度
            watermark.Rotation = -15;

            // Remove the following two lines if you need a solid black text.
            // 控制水印的颜色
            watermark.Fill.Color = Color.Gray; // Try LightGray to get more Word-style watermark
            watermark.StrokeColor = Color.Gray; // Try LightGray to get more Word-style watermark

            // 水平居中
            watermark.RelativeHorizontalPosition = RelativeHorizontalPosition.Page;

            // 垂直居中 注释后会在word正文的头部显示
            //watermark.RelativeVerticalPosition = RelativeVerticalPosition.Page;

            watermark.WrapType = WrapType.None;

            // 文字的居中方式
            watermark.VerticalAlignment = VerticalAlignment.Center;
            watermark.HorizontalAlignment = HorizontalAlignment.Center;

            // Create a new paragraph and append the watermark to this paragraph.

            Paragraph watermarkPara = new Paragraph(doc);

            watermarkPara.AppendChild(watermark);

            // Insert the watermark into all headers of each document section.

            foreach (Section sect in doc.Sections)
            {
                // There could be up to three different headers in each section, since we want
                // the watermark to appear on all pages, insert into all headers.
                InsertWatermarkIntoHeader(watermarkPara, sect, HeaderFooterType.HeaderPrimary);
                InsertWatermarkIntoHeader(watermarkPara, sect, HeaderFooterType.HeaderFirst);
                InsertWatermarkIntoHeader(watermarkPara, sect, HeaderFooterType.HeaderEven);
            }
        }

        public void InsertWatermarkIntoHeader(Paragraph watermarkPara, Section sect, HeaderFooterType headerType)
        {
            HeaderFooter header = sect.HeadersFooters[headerType];
            if (header == null)
            {
                // There is no header of the specified type in the current section, create it.
                header = new HeaderFooter(sect.Document, headerType);
                sect.HeadersFooters.Add(header);
            }
            // Insert a clone of the watermark into the header.
            header.AppendChild(watermarkPara.Clone(true));
        }

        #endregion

        /// <summary>
        /// 1.6 输出表格
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void OutPutTable_Click(object sender, EventArgs e)
        {
            var path = Server.MapPath("../Document/AsposeWord.doc");
            Document doc = new Document(path);
            DocumentBuilder builder = new DocumentBuilder(doc);

            Aspose.Words.Font font = builder.Font;
            font.Bold = false;
            font.Size = 12;

            // 设置表格的样式
            builder.RowFormat.Borders.LineStyle = LineStyle.Single;
            builder.RowFormat.HeightRule = HeightRule.Exactly;
            builder.RowFormat.Alignment = RowAlignment.Center;
            // 设置行高
            builder.RowFormat.Height = 24; // 设置单元格的高度

            // 表格设置为4列 总宽度为492.7
            double width1 = 92.7;
            double width2 = 100;
            double width3 = 150;
            double width4 = 150;

            // 设置每个单元格内的样式
            builder.CellFormat.VerticalAlignment = CellVerticalAlignment.Center;
            builder.ParagraphFormat.Alignment = ParagraphAlignment.Center;
            builder.CellFormat.VerticalMerge = CellMerge.First;
            builder.CellFormat.HorizontalMerge = CellMerge.None;

            // 开始画单元格
            builder.InsertCell();
            // 设置单元格的宽度
            builder.CellFormat.Width = width1;
            // 单元格中的内容
            builder.Write("姓名");

            builder.InsertCell();
            builder.CellFormat.Width = width2;
            builder.Write("证件类型");

            builder.InsertCell();
            builder.CellFormat.Width = width3;
            builder.Write("证件号码");

            builder.InsertCell();
            builder.CellFormat.Width = width4;
            builder.Write("联系方式");

            // 每一行结束一定要EndRow() 不然每一行的样式会受到影响
            builder.EndRow();

            // 输出4行空表格
            for (var i = 0; i < 4; i++)
            {
                builder.InsertCell();
                builder.CellFormat.Width = width1;

                builder.InsertCell();
                builder.CellFormat.Width = width2;

                builder.InsertCell();
                builder.CellFormat.Width = width3;

                builder.InsertCell();
                builder.CellFormat.Width = width4;

                builder.EndRow();
            }

            var fileName = "OutPutTable(" + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss") + ").doc";
            doc.Save(fileName, SaveFormat.Doc, SaveType.OpenInBrowser, Response);
        }
        #endregion


        #region 2.0-综合实例介绍

        /// <summary>
        /// 2.1 运用书签输出表格
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void OutPutTableWidthBookMark_Click(object sender, EventArgs e)
        {
            var path = Server.MapPath("../Document/Aspose.WordBookMark.doc");
            Document doc = new Document(path);
            DocumentBuilder builder = new DocumentBuilder(doc);

            Aspose.Words.Font font = builder.Font;
            font.Bold = false;
            font.Size = 12;

            // 设置表格的样式
            builder.RowFormat.Borders.LineStyle = LineStyle.Single;
            builder.RowFormat.HeightRule = HeightRule.Exactly;
            builder.RowFormat.Alignment = RowAlignment.Center;
            // 设置行高
            builder.RowFormat.Height = 24; // 设置单元格的高度

            // 设置每个单元格内的样式
            builder.CellFormat.VerticalAlignment = CellVerticalAlignment.Center;
            builder.ParagraphFormat.Alignment = ParagraphAlignment.Center;
            builder.CellFormat.VerticalMerge = CellMerge.First;
            builder.CellFormat.HorizontalMerge = CellMerge.None;

            // 定义一个存储模板表头每个单元格宽度的集合
            List<double> cellWidths = new List<double>(); 
            // 根据集合列数
            for (int i = 0; i < 4; i++)
            {
                // 参数依次是： "表格的索引"、"行的索引"、"列的索引"、"单元格内部索引"
                builder.MoveToCell(0, 0, i, 0); //移动单元格 （代表着表头的cell依次移动）
                double width = builder.CellFormat.Width;//获取单元格宽度
                cellWidths.Add(width);
            }

            // 直接定位到书签的位置 （根据书签名定位的）
            builder.MoveToBookmark("UserInfoTable");
            // 输出表格
            for (var i = 0; i < 6; i++)
            {
                builder.InsertCell();
                // 设置单元格宽度
                builder.CellFormat.Width = cellWidths[0];
                builder.Write("张三"+i);

                builder.InsertCell();
                builder.CellFormat.Width = cellWidths[1];
                builder.Write("身份证" + i);

                builder.InsertCell();
                builder.CellFormat.Width = cellWidths[2];
                builder.Write("ABC" + i);

                builder.InsertCell();
                builder.CellFormat.Width = cellWidths[3];
                builder.Write("888" + i);

                builder.EndRow();
            }
            // 结束表格
            builder.EndTable();
            // 结束书签
            builder.EndBookmark("UserInfoTable"); 

            var fileName = "OutPutTableWidthBookMark(" + DateTime.Now.ToString("yyyy-MM-dd HH-mm-ss") + ").doc";
            doc.Save(fileName, SaveFormat.Doc, SaveType.OpenInBrowser, Response);
        }
        #endregion
    }
}